@@ -12,9 +12,6 @@ import texttable
 import tools
 import getpass
 import traceback
-import readline
-
-
 
 d_type = "postgresql"
 d_host = ""
@@ -166,12 +163,24 @@ class associateCmd(cmd.Cmd):
 	def __init__(self):
 		cmd.Cmd.__init__(self)
 		#self.var_list = set()
-		import readline
-		readline.set_history_length(80)
 		try:
-                    readline.read_history_file()
-                except IOError:
-                    readline.write_history_file()
+			import readline
+			readline.set_history_length(80)
+			try:
+       			        readline.read_history_file()
+                	except IOError:
+                		readline.write_history_file()
+		except ImportError:
+			try : 
+				import pyreadline as readline		
+				readline.set_history_length(80)
+				try:
+       				        readline.read_history_file()
+             		   	except IOError:
+                			readline.write_history_file()
+			except ImportError:
+				pass
+		
 	def completedefault(self, text, line, begidx, endidx):
 		if not text:
 			completions = self.assocword[:]
@@ -191,25 +200,39 @@ class associateCmd(cmd.Cmd):
 			cmd_line=line[line.find('=') + 1:]
 		else:
 			cmd_line = line
+                if "FIND" in line:
+                    line=line.replace("FIND","find")
+                elif "QUERY" in line:
+                    line=line.replace("QUERY","query")
+                elif "SHOW" in line:
+                    line=line.replace("SHOW","show")
+                elif "ROW" in line:
+                    line=line.replace("ROW","row")
+                elif "COLUMN" in line:
+                    line=line.replace("COLUMN","column")
+
+
 
 		if line.strip().replace(" ","").find("=find") != -1: 
                         if "where " in line:
-       	                     where_str_origin=cmd_line[cmd_line.find("where")+5:]
-                             tmp_split=where_str_origin.split("=")
-                             prefixStr=str()
-                             varStr=str()
-                             if len(tmp_split)>=2:
-                                  prefixStr=tmp_split[0]+"="
-                                  varStr=tmp_split[1]
-                                  where_str=self.replace_var(varStr,prefix=prefixStr,joinstr="or")
-                                  cmd_line=cmd_line.replace(where_str_origin,where_str)               
-             
+                             where_str_origin=cmd_line[cmd_line.find("where")+5:]
+                             cmd_line=cmd_line.replace(where_str_origin,self.cond_parser(where_str_origin,[" AND "," OR "," and "," or "]))
                         
 			return_data = rpcshell.shell(self.host, self.port ,self.token, "assoc" , cmd_line, self.timeout)
 			globals()[var_name] = return_data
+
+		elif line.strip().replace(" ","").find("=show") != -1: 
+                        if "where" in line:
+       	                        start=cmd_line.find(":",cmd_line.find("where"))
+                                end=cmd_line.find(";",start)
+                                where_str_origin=cmd_line[start+1:end]
+                                print where_str_origin
+                      		cmd_line=cmd_line.replace(where_str_origin,self.cond_parser(where_str_origin,[" AND "," OR "," and "," or "]))
+			return_data = rpcshell.shell(self.host, self.port ,self.token, "assoc" , cmd_line, self.timeout)
+			globals()[var_name] = return_data
 		
 		elif line.strip().replace(" ","").find("=query") != -1: 
-			#if cmd_line.find('$') != -1:
+			#if cmd_line.find('$') != -1
 			#	var_va = cmd_line[cmd_line.find('$') + 1:cmd_line.find(' ',cmd_line.find('$'))]
 			#	cmd_line = cmd_line.replace("$"+var_va , str(globals()[var_va])[1:-1])
 			#	#print line
@@ -284,34 +307,35 @@ class associateCmd(cmd.Cmd):
 	#	for var_temp in self.get_bosh_global_var_with_filter():
 	#		if var_temp in line:
 	#			print "found"
-
+        def do_LISTVAR(self, line):
+                self.do_listvar(line)
 	
 	def do_listvar(self, line):
 		#print "all variables: " + str(list(self.var_list))
 		print "all variables: "		
 		print self.get_bosh_global_var_with_filter()
+
+        def do_FIND(self, line):
+                self.do_find(line)
 	def do_find(self, line):
+                if "WHERE" in line:
+                      line.replace("WHERE","where")
                 if "where " in line:
 	              where_str_origin=line[line.find("where")+5:]
-                      tmp_split=where_str_origin.split("=")
-                      prefixStr=str()
-                      varStr=str()
-                      if len(tmp_split)>=2:
-                           prefixStr=tmp_split[0]+"="
-                           varStr=tmp_split[1]
-                           where_str=self.replace_var(varStr,prefix=prefixStr,joinstr="or")
-                           line=line.replace(where_str_origin,where_str)               
-                
+                      line=line.replace(where_str_origin,self.cond_parser(where_str_origin,[" AND "," OR "," and "," or "]))
+            
          	rpcshell.shell(self.host, self.port ,self.token, "assoc" , "find " + line, self.timeout)
-
-
+        
+        def do_CREATE(self, line):
+                self.do_create(line)
 	def do_create(self, line):
 		#print rpcshell.shell(self.host, self.port ,self.token, "assoc" , "build " + line, self.timeout)
 		print rpcshell.shell(self.host, self.port ,self.token, "assoc" , "create " + line, self.timeout)
-
+        def do_BUILD(self, line):
+                self.do_build(line)
 	def do_build(self, line):
 		#print rpcshell.shell(self.host, self.port ,self.token, "assoc" , "build " + line, self.timeout)
-		print rpcshell.shell(self.host, self.port ,self.token, "assoc" , "create " + line, self.timeout)
+		print rpcshell.shell(self.host, self.port ,self.token, "assoc" , "build " + line, self.timeout)
         
         def do_ask(self, line):
                 if "about" in line:
@@ -341,15 +365,34 @@ class associateCmd(cmd.Cmd):
 	#			self.algorithm = line[10:]
 	#			print "algorithm : " + self.algorithm
 
+        def do_SETPROMPT(self, line):
+                self.prompt = line
 	def do_setprompt(self, line):
 		self.prompt = line
+
+        def do_BOURL(self,line):
+                self.do_bourl(line)
+        def do_bourl(self,line):
+             	if self.bosrv_url != line:
+		   url_object = urlparse(line)
+                   self.bosrv_url=line
+		   self.token = url_object.password if url_object.password !=None else self.token
+		   self.host = url_object.hostname if url_object.hostname !=None else self.host
+		   self.port = url_object.port if url_object.port !=None else self.port
+
+        def do_INFO(self, line):   
+                self.do_info(line)
 	def do_info(self, line):
 		print "host : " + self.host	
 		print "port : " + str(self.port)
 		print "timeout : " + str(self.timeout)
 		#print "algorithm : " + self.algorithm
+   
+        def do_SHOW(self,line):
+                self.do_show(line)
+
 	def do_show(self, line):
-		if line == "tables":
+		if line == "tables" or  line=="TABLES":
 			print boshCmd.getinfo(self.host, self.port ,self.token, "get_table_list" , "" , self.timeout)
 		#elif line == "matrices":
 		#	print boshCmd.getinfo(self.host, self.port ,self.token, "get_mbt_list" , "" , self.timeout)
@@ -358,7 +401,14 @@ class associateCmd(cmd.Cmd):
 		elif line == "association":
 			print boshCmd.getinfo(self.host, self.port ,self.token, "get_qbo_list" , "" , self.timeout)
 		else:
-			print "show [tables | association ]"
+			if "where" in line:
+                                start=line.find(":",line.find("where"))
+                                end=line.find(";",start)
+                                where_str_origin=line[start+1:end]
+                      		line=line.replace(where_str_origin,self.cond_parser(where_str_origin,[" AND "," OR "," and "," or "]))
+         		rpcshell.shell(self.host, self.port ,self.token, "assoc" , "show " + line, self.timeout)
+        def do_DESC(self, line):
+                self.do_desc(line)
 	def do_desc(self, line):
 		if line[:12] == "association ":
 			print boshCmd.getinfo(self.host, self.port ,self.token, "get_qbo_info" , line[12:], self.timeout)
@@ -393,10 +443,13 @@ class associateCmd(cmd.Cmd):
 		return True	
 	def emptyline(self):
     		pass
+
+        def do_SQL(self, line):
+                self.do_sql(line)
 	def do_sql(self, line):
 		if line != "":
 			print "perform sql command : " + line
-			print rpcshell.shell(self.host, self.port ,self.token, "sql" , line, self.timeout)
+			rpcshell.shell(self.host, self.port ,self.token, "sql" , line, self.timeout)
 		else:
 			newcmd = sqlCmd()
 			newcmd.host = self.host
@@ -406,6 +459,8 @@ class associateCmd(cmd.Cmd):
 			newcmd.prompt = self.prompt[:len(self.prompt)-1] + ":*sql>"
 			newcmd.draw_texttable()
 			newcmd.cmdloop()
+        def do_ADMIN(self, line):
+                self.do_admin(line)
 	def do_admin(self, line):
 		newcmd = baseCmd()
 		newcmd.host = self.host
@@ -415,6 +470,8 @@ class associateCmd(cmd.Cmd):
 		newcmd.prompt = self.prompt[:len(self.prompt)-1] + ":admin>"
 		newcmd.cmdloop()
 
+        def do_PRINT(self,line):
+               self.do_print(line)
         def do_print(self,line):
                command=line.split(">>")
                if len(command)==2:
@@ -441,18 +498,71 @@ class associateCmd(cmd.Cmd):
 
                        
 	def writehist(self):
-		import readline
-		readline.set_history_length(80)
-		readline.write_history_file()
+		try:
+  			import readline
+			readline.set_history_length(80)
+			readline.write_history_file()
+		except ImportError:
+			try:
+  				import pyreadline as readline		
+				readline.set_history_length(80)
+				readline.write_history_file()
+			except ImportError:
+				pass
 		
 	def get_bosh_global_var_with_filter(self):
 		f = ['re', 'readline', 'd_pass', 'cmd', 'getpass', 'rpcshell', 'd_port', 'tools', 'dbinput', 'd_host', 'csvloader', 'load_database_url', 'main', 'sys', 'd_db', 'sqlCmd', 'texttable', 'baseCmd', 'associateCmd', 'traceback', 'urlparse', 'd_user', 'os', 'd_type', 'db_access_str', 'boshCmd']
 		return [v for v in globals().keys() if not v.startswith('_') and not v in f]
 
+        def cond_parser(self,ori_str,kwlist):
+                stackdict=self.stack_parser(ori_str)
+                cond_list=list();
+                if(len(stackdict)!=0):
+                      cond_key=max(stackdict,key=int)
+                      cond_list=stackdict[cond_key]
+                else:
+                      cond_list=[ori_str]
+                new_str=ori_str
+                for cond_str in cond_list:
+                      tmp_str=self.replace_cond(cond_str,kwlist)
+                      new_str=new_str.replace(cond_str,tmp_str)
+
+                return new_str
+
+        def replace_cond(self,cond_str,kwlist):
+                pattern="|".join(kwlist)
+                atomcond_list=re.split(pattern,cond_str)
+                new_str=cond_str
+                for atomcond_str in atomcond_list:
+                       tmp_split=atomcond_str.split("=") 
+                       if len(tmp_split)>=2:
+                           prefixStr=tmp_split[0]+"="
+                           varStr=tmp_split[1]
+                           tmp_str=self.replace_var(varStr,prefix=prefixStr,joinstr="or")
+                           new_str=new_str.replace(atomcond_str,tmp_str)
+                
+                return new_str
+                                             
+             
+        def stack_parser(self,ori_str):
+                stackdict=dict();
+                stack=[];
+                for i, char in enumerate(ori_str):
+                     if char=="(":
+                         stack.append(i)
+                     elif char==")":
+                         start=stack.pop()
+                         depth=len(stack)
+                         token=ori_str[start+1:i]
+                         if depth not in stackdict:
+                               stackdict[depth]=[]
+                         stackdict[depth].append(token)
+                return stackdict
+
         def replace_var(self,ori_str,prefix="",joinstr=","):
                 #print "input str: ", ori_str
                 #print "prefix: ", prefix
-                
+                joinstr=" "+joinstr+" "
          	split_temp = re.split(r'[,]', ori_str)
                 result_list=list()
                 replace_list=list()
@@ -461,15 +571,21 @@ class associateCmd(cmd.Cmd):
                         split_value=split_value.strip()
                         if split_value=="":continue
 			if split_value in self.get_bosh_global_var_with_filter():
-                                if isinstance(globals()[split_value],basestring):
-                                     replace_list.append(globals()[split_value])
-                                else:       
-                                     replace_list.extend(globals()[split_value])
+                               if  hasattr(globals()[split_value], '__iter__'):
+                                      replace_list.extend(globals()[split_value])
+                               else:
+                                      replace_list.append(globals()[split_value])
+                                       
                         else:
                                 result_list.append(split_value)
                 #print "result var list:",result_list 
-                new_str=joinstr.join(prefix+str(element) for element in result_list)
-                new_str+=joinstr.join(prefix+"'"+str(element)+"'" for element in replace_list)
+                new_str=joinstr.join(prefix+str(element)+" " for element in result_list)
+                #new_str+=joinstr.join(prefix+"'"+str(element)+"'" for element in replace_list)
+                quote=str()
+                if len(replace_list)>0 :
+                        if isinstance(replace_list[0],basestring):
+                             quote="'"
+                new_str+=joinstr.join(" "+prefix+quote+str(element)+quote for element in replace_list)
                 
                 #print "final str:" ,new_str
                 return new_str
@@ -545,8 +661,10 @@ class sqlCmd(cmd.Cmd):
 		print "server : " + self.host + ":" + str(self.port)
 		print "enter quit/exit to exit bosh"
 
+        def do_SHOW(self, line):
+                self.do_show(line)
 	def do_show(self, line):
-		if line == "tables":
+		if line == "tables" or line == "TABLES":
 			print boshCmd.getinfo(self.host, self.port ,self.token, "get_table_list" , "" , self.timeout)
 		else:
 			print "list tables: 'show tables'"
@@ -554,6 +672,7 @@ class sqlCmd(cmd.Cmd):
 	def do_starschema(self, line):
 		if line != "":
 			print boshCmd.getinfo(self.host, self.port ,self.token, "get_table_relation" , line , self.timeout)
+
 	def do_desc(self, line):
 		if line != "":
 			print boshCmd.getinfo(self.host, self.port ,self.token, "get_table_schema" , line, self.timeout)
@@ -565,28 +684,45 @@ class sqlCmd(cmd.Cmd):
 		print "host : " + self.host	
 		print "port : " + str(self.port)
 		print "timeout : " + str(self.timeout)
+ 
+        def do_SELECT(self, line):
+                self.do_select(line)
 	def do_select(self, line):
 		if line != "":
 			if line.startswith('distinct'):
 				print rpcshell.shell(self.host, self.port ,self.token, "la" , "select " + line, self.timeout)
 			else:
 				print rpcshell.shell(self.host, self.port ,self.token, "sql" , "select " + line, self.timeout)
+        def do_CREATE(self,line):
+                self.do_create(line)
 	def do_create(self, line):
 		if line != "":
 			if line.startswith('tree') or line.startswith('fulltree'):
 				print rpcshell.shell(self.host, self.port ,self.token, "la" , "create " + line, self.timeout)
 			else:
 				print rpcshell.shell(self.host, self.port ,self.token, "sql" , "create " + line, self.timeout)	
+
+        def do_INSERT(self, line):
+                self.do_insert(line)
 	def do_insert(self, line):
 		if line != "":
 			print rpcshell.shell(self.host, self.port ,self.token, "sql" , "insert " + line, self.timeout)	
+
+        def do_UPDATE(self, line):
+                self.do_update(line)
 	def do_update(self, line):
 		if line != "":		
 			print rpcshell.shell(self.host, self.port ,self.token, "sql" , "update " + line, self.timeout)	
+
+        def do_DROP(self, line):
+                self.do_drop(line)
 	def do_drop(self, line):
 		if line != "":
 			print rpcshell.shell(self.host, self.port ,self.token, "sql" , "drop " + line, self.timeout)	
 
+
+        def do_COPY(self, line):
+                self.do_copy(line)
 	def do_copy(self, line):
 		db_url = os.environ.get('DATABASE_URL')
 		db_access_str = load_database_url(db_url)
@@ -600,6 +736,9 @@ class sqlCmd(cmd.Cmd):
 				db_access_str = dbinput()
 				line = line.replace("DATABASE" , db_access_str)
 			print rpcshell.shell(self.host, self.port ,self.token, "sql" ,"copy " + line , self.timeout) 	
+
+        def do_LOAD(self, line):
+                self.do_load(line)
 	def do_load(self, line):
 		db_url = os.environ.get('DATABASE_URL')
 		db_access_str = load_database_url(db_url)
