@@ -82,7 +82,7 @@ class C3(BaseHtml,object):
            strDocReady='var data={};\n'.format(str(data))
            strDocReady+='var bo=BigObject_D3Chart;\n\n\n'
            for bindID,chartType in kwargs.items():
-                  strDocReady += 'bo.draw_LocalData(data,"{}","#{}");\n'.format(chartType,bindID)       
+                  strDocReady += 'bo.draw_LocalData(data,"{}","#{}",600);\n'.format(chartType,bindID)       
                   bindID
 
 
@@ -106,9 +106,8 @@ class C3(BaseHtml,object):
      @htmlTag
      def _genHTML(self,jsonData,chartType):
            chartDIV=self._createDiv(""" id="chart1" """)
-           pieDIV=self._createDiv(""" id="defaultPie" """)
-           strDIV=self._createDiv(""" class="containerLeft" """,divElements=chartDIV+pieDIV)
-           listChart={"chart1":chartType,"defaultPie":"pie"}
+           strDIV=self._createDiv(""" class="containerLeft" """,divElements=chartDIV)
+           listChart={"chart1":chartType}
            strJS=self._loadJSDocReady(jsonData,**listChart)
         
            html_string=str()
