@@ -136,11 +136,18 @@ def Create():
                                                                      )
                         ,"\n\t\tex."
                         ,"\t\t{0} {1} {2}".format(cmd("create table"),
-                                                  arguments("log.bt"),
+                                                  arguments("log"),
                                                   cmd("(")+arguments("session_id INT64,url STRING")+cmd(")")
                                                  )
+			,"\t\tex. create a dim table"
+                        ,"\t\t{0} {1} {2}".format(cmd("create table"),arguments("Product"),cmd("("))
+                        ,"\t\t                     {0}".format(arguments("id STRING,"))
+                        ,"\t\t                     {0}".format(arguments("name STRING,"))
+                        ,"\t\t                     {0}".format(arguments("brand STRING,"))
+			,"\t\t                     {0}".format(arguments("KEY(id)"))
+                        ,"\t\t                     {0}".format(cmd(")"))
                         ,"\t\tex. create a fact table"
-                        ,"\t\t{0} {1} {2}".format(cmd("create table"),arguments("Fact.bt"),cmd("("))
+                        ,"\t\t{0} {1} {2}".format(cmd("create table"),arguments("Fact"),cmd("("))
                         ,"\t\t                     {0}".format(arguments("Customer.id STRING,"))
                         ,"\t\t                     {0}".format(arguments("Product.id STRING,"))
                         ,"\t\t                     {0}".format(arguments("date DATETIME64,"))
