@@ -12,16 +12,15 @@ def shell(host, port, token_str, shell_name, command, timeout):
 	now = time.time()
 	result_str = ""
 	addr_info= "bo://:" + token_str + "@" + host + ":" + str(port)
-	
-        temp_command_str=command.split('>>')
-        output_command=str()
-
-        if len(temp_command_str)>=2:
-              command=temp_command_str[0]
-              output_command=temp_command_str[1]         
-              if len(output_command)==0:
-                   return "*** FILENAME required after \">>\""           
-
+        parsedict=tools.postactionParser(command)
+        if parsedict["ERR_msg"]!=None:
+              print parsedict["ERR_msg"]
+              return parsedict["ERR_msg"]
+ 
+        command=parsedict["command"]
+        out_command=parsedict["out_command"]
+        pipes_command=parsedict["pipes_command"]
+        doInvoke=parsedict["doInvoke"]
 	result_table = []
 
 	try:
@@ -76,23 +75,17 @@ def shell(host, port, token_str, shell_name, command, timeout):
 				    #table_str += str(record) + ","
 				result_str += "size:" + str(len(result_table)) + "\n"
                                 
+                                if pipes_command !=None:
+                                     tools.runPipes(result_table,pipes_command)
  
-		                if output_command :
-                                     doInvoke=False
-                                     outfile=str()
-                                     if "@" in output_command:
-                                            outfile=output_command[output_command.find("@")+1:].strip()
-                                            if len(outfile)==0:
-                                                   return "*** FILENAME required after \">>@\""
-                                            doInvoke=True
-                                     else:
-                                            outfile=output_command.strip()
-                                     outfile=tools.redirectFiles(result_table,outfile) 
+		                if out_command !=None:
+                                     outfile=tools.redirectFiles(result_table,out_command) 
                                      if outfile=="BADINPUT":
                                             return "*** cancel outputting the file"
                                      if doInvoke:
                                             tools.invokeFiles(outfile)	
 
+
 			#print "send: \"" + command + "\" to " + host + ":" + str(port) + "\n" + result_str + '\ntime: %ss' %  str(round((end - now),2)) + "\n" 	
 			print "send: \"" + command + "\" to " + host + ":" + str(port) + "\n"
 			print result_str 
