@@ -2,6 +2,8 @@ import boshC3
 import boshExcel
 import docSql
 import os
+import json
+import sys
 def psql_run(d_host, d_port, d_user, d_pass, d_db):
 	import getpass
 	import os
@@ -185,8 +187,25 @@ def docSqlInsert():
 
 
 
-def redirectFiles(result_table,outfile):
+def dump2Csv(result_table,outfile,line_count,status="append"):
+         outfile=outfile if outfile!=None else "dump_result.csv"
+         if ".csv" not in outfile:
+              outfile+=".csv"
+         import csv
+         writemode="w" if status=="init" else "a"
+         local_count=line_count
+         with open(outfile,writemode) as csvfile:        
+                writer=csv.writer(csvfile)
+                for row in result_table:
+                         writer.writerow([unicode(s).encode("utf-8") for s in row])
+                         local_count+=1
+                         sys.stdout.write("\r")
+                         sys.stdout.write("dump "+str(local_count)+" lines")
+                         sys.stdout.flush()
 
+         return outfile
+
+def redirectFiles(result_table,outfile):
          if ".html" in outfile:                                          
                 outfile=boshC3.Default_output(result_table,outfile)
          elif ".xlsx" in outfile:
@@ -197,7 +216,7 @@ def redirectFiles(result_table,outfile):
                 with open(outfile,"w") as csvfile:
                      writer=csv.writer(csvfile)
                      for row in result_table:
-                            writer.writerow(row)    
+                            writer.writerow([unicode(s).encode("utf-8") for s in row])    
 	
          else:
                 fd=open(outfile,"w")
@@ -225,3 +244,58 @@ def invokeFiles(outfile):
          finally:
              # os.dup2(saverr,2)
 	      pass                                                        
+
+
+
+def postactionParser(command_str):
+         parsedict={"command":None,"out_command":None,"pipes_command":None,"ERR_msg":None,"doInvoke":False}
+         parsedict["command"]=command_str
+
+         splits=command_str.split(">>")
+         if len(splits)>=2:
+              if len(splits[1])==0:
+                   parsedict["ERR_msg"]="*** FILENAME required after \">>\""
+                   return parsedict
+              elif "|" in splits[1]:
+                   parsedict["ERR_msg"]="*** Operations via PIPES should be put before \">>\""
+                   return parsedict
+              elif "@" in splits[1]:
+                   outfile=splits[1][splits[1].find("@")+1:].strip()
+                   if len(outfile)==0:
+                        parsedict["ERR_msg"]="*** FILENAME required after \">>@\""
+                        return parsedict 
+                   parsedict["doInvoke"]=True
+                   parsedict["out_command"]=outfile
+              else:
+                   parsedict["out_command"]=splits[1].strip()
+              
+              parsedict["command"]=splits[0].strip()
+
+
+         splits=parsedict["command"].split("|")  
+         if len(splits)>=2:
+              if len(splits[1])==0:
+                   parsedict["ERR_msg"]="*** PROGRAM/COMMAND required after \"|\""
+                   return parsedict
+              strBegin=parsedict["command"].find("|")
+              parsedict["pipes_command"]=parsedict["command"][strBegin+1:]
+              parsedict["command"]=splits[0]
+           
+             
+
+         return parsedict
+
+
+
+def runPipes(result_table,pipes_command):
+        from subprocess import Popen, PIPE
+        jsonstr=json.dumps(result_table)
+        command=pipes_command.split()
+        
+        process=Popen(command,stdin=PIPE)  #use STDIN to send data, could use linux build-in commands easily
+        process.stdin.write(jsonstr)
+
+        #command.append(jsonstr)    # append data (jsonstr) as the last arguments 
+        #process=Popen(command)
+
+        process.communicate() 
